@@ -33,7 +33,7 @@
 
 #if 0
 static const char *version =
-    "cardinfo.c %revision% %date% (David Hinds)";
+"cardinfo.c 1.10 1996/08/06 05:07:07 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -67,7 +67,7 @@ static socket_info_t st[MAX_SOCK];
 static FL_OBJECT *event_log;
 
 static char *pidfile = "/var/run/cardmgr.pid";
-static char *stabfile = "/etc/stab";
+static char *stabfile = "/var/run/stab";
 
 /*====================================================================*/
 
@@ -86,6 +86,7 @@ static event_tag_t event_tag[] = {
     { CS_EVENT_INSERTION_REQUEST, "user insert request" },
     { CS_EVENT_PM_SUSPEND, "suspend card" },
     { CS_EVENT_PM_RESUME, "resume card" },
+    { CS_EVENT_REQUEST_ATTENTION, "request attention" },
 };
 #define NTAGS (sizeof(event_tag)/sizeof(event_tag_t))
 
@@ -154,14 +155,16 @@ static void do_menu(FL_OBJECT *obj, long i)
 
     switch (fl_get_menu(obj)) {
     case 1:
-	ret = ioctl(st[i].fd, DS_RESET_CARD); break;
+	/* do_opts(); */ break;
     case 2:
-	ret = ioctl(st[i].fd, DS_SUSPEND_CARD); break;
+	ret = ioctl(st[i].fd, DS_RESET_CARD); break;
     case 3:
-	ret = ioctl(st[i].fd, DS_RESUME_CARD); break;
+	ret = ioctl(st[i].fd, DS_SUSPEND_CARD); break;
     case 4:
-	ret = ioctl(st[i].fd, DS_EJECT_CARD); break;
+	ret = ioctl(st[i].fd, DS_RESUME_CARD); break;
     case 5:
+	ret = ioctl(st[i].fd, DS_EJECT_CARD); break;
+    case 6:
 	ret = ioctl(st[i].fd, DS_INSERT_CARD); break;
     }
     if (ret != 0)
@@ -244,7 +247,7 @@ static void do_update(FL_OBJECT *obj, long data)
     FILE *f;
     int i, j, event, ret, state;
     status_t status;
-    config_t cfg;
+    config_info_t cfg;
     char c, s[80], *t, d[80], io[20], irq[4];
     ioaddr_t stop;
     struct stat buf;
@@ -283,7 +286,7 @@ static void do_update(FL_OBJECT *obj, long data)
     }
 
     if ((stat(stabfile, &buf) == 0) && (buf.st_mtime >= last)) {
-	f = fopen("/etc/stab", "r");
+	f = fopen(stabfile, "r");
 	if (f == NULL)
 	    return;
 	
@@ -306,8 +309,9 @@ static void do_update(FL_OBJECT *obj, long data)
 		}
 		else {
 		    fgets(s, 80, f);
-		    t = strchr(strchr(s, '\t')+1, '\t')+1;
-		    *strchr(t, '\t') = '\0';
+		    for (t = s, j = 0; j < 4; j++)
+			t = strchr(t, '\t')+1;
+		    t[strcspn(t, "\t\n")] = '\0';
 		    if (*d == '\0')
 			strcpy(d, t);
 		    else {
@@ -342,31 +346,34 @@ static void do_update(FL_OBJECT *obj, long data)
 	
 	if (state != st[i].o_state) {
 	    st[i].o_state = state;
-	    for (j = 1; j <= 5; j++)
+	    for (j = 1; j <= 6; j++)
 		fl_set_menu_item_mode(st[i].menu, j, FL_PUP_GRAY);
 	    switch (state) {
 	    case S_EMPTY:
 		update_field(&st[i].state, "");
 		break;
 	    case S_PRESENT:
-		fl_set_menu_item_mode(st[i].menu, 5, FL_PUP_NONE);
+		fl_set_menu_item_mode(st[i].menu, 6, FL_PUP_NONE);
 		update_field(&st[i].state, "");
 		break;
 	    case S_READY:
 		fl_set_menu_item_mode(st[i].menu, 1, FL_PUP_NONE);
 		fl_set_menu_item_mode(st[i].menu, 2, FL_PUP_NONE);
-		fl_set_menu_item_mode(st[i].menu, 4, FL_PUP_NONE);
+		fl_set_menu_item_mode(st[i].menu, 3, FL_PUP_NONE);
+		fl_set_menu_item_mode(st[i].menu, 5, FL_PUP_NONE);
 		update_field(&st[i].state, "ready");
 		break;
 	    case S_BUSY:
 		fl_set_menu_item_mode(st[i].menu, 1, FL_PUP_NONE);
 		fl_set_menu_item_mode(st[i].menu, 2, FL_PUP_NONE);
-		fl_set_menu_item_mode(st[i].menu, 4, FL_PUP_NONE);
+		fl_set_menu_item_mode(st[i].menu, 3, FL_PUP_NONE);
+		fl_set_menu_item_mode(st[i].menu, 5, FL_PUP_NONE);
 		update_field(&st[i].state, "not ready");
 		break;
 	    case S_SUSPEND:
-		fl_set_menu_item_mode(st[i].menu, 3, FL_PUP_NONE);
+		fl_set_menu_item_mode(st[i].menu, 1, FL_PUP_NONE);
 		fl_set_menu_item_mode(st[i].menu, 4, FL_PUP_NONE);
+		fl_set_menu_item_mode(st[i].menu, 5, FL_PUP_NONE);
 		update_field(&st[i].state, "suspended");
 		break;
 	    }
@@ -374,8 +381,9 @@ static void do_update(FL_OBJECT *obj, long data)
 
 	strcpy(io, "");
 	strcpy(irq, "");
+	cfg.Function = 0;
 	ret = ioctl(st[i].fd, DS_GET_CONFIGURATION_INFO, &cfg);
-	if (ret == 0) {
+	if (cfg.Attributes & CONF_VALID_CLIENT) {
 	    if (cfg.AssignedIRQ != 0)
 		sprintf(irq, "%ld", cfg.AssignedIRQ);
 	    if (cfg.NumPorts1 > 0) {
@@ -451,8 +459,14 @@ void main(int argc, char *argv[])
 	perror("forking");
     if (setsid() < 0)
 	perror("detaching from tty");
-    
+
+#if (FL_REVISION >= 80)
+    fl_flip_yorigin();
+    fl_initialize(&argc, argv, "cardinfo", 0, 0);
+#else
     fl_initialize(argv[0], "cardinfo", 0, 0, &argc, argv);
+#endif
+    
     form = fl_bgn_form(FL_BORDER_BOX, 400, ns*100+70);
     
     for (i = 0; i < ns; i++) {
@@ -463,7 +477,7 @@ void main(int argc, char *argv[])
 	fl_set_object_boxtype(obj, FL_UP_BOX);
 	fl_set_object_callback(obj, do_menu, i);
 	fl_show_menu_symbol(obj, 1);
-	fl_set_menu(obj, "reset|suspend|resume|eject|insert");
+	fl_set_menu(obj, "opts...|reset|suspend|resume|eject|insert");
 	new_field(&st[i].card, 110, y, 0, 280, "");
 
 	y -= 25;
@@ -474,7 +488,7 @@ void main(int argc, char *argv[])
 	new_flag(&st[i].wp, 360, y, "WP");
 	
 	y -= 25;
-	new_field(&st[i].dev, 110, y, 60, 100, "device(s):");
+	new_field(&st[i].dev, 110, y, 60, 160, "device(s):");
 
 	y -= 25;
 	new_field(&st[i].io, 110, y, 50, 150, "IO ports:");
