@@ -25,15 +25,27 @@
 #include <sys/file.h>
 
 #include "version.h"
+#include "config.h"
 #include "cs_types.h"
 #include "cs.h"
 #include "cistpl.h"
 #include "ds.h"
 
+#define VERSION(v,p,s) (((v)<<16)+(p<<8)+s)
+#if (LINUX_VERSION_CODE > VERSION(1,3,0))
+#define GET_SCSI_INFO
+#include <linux/major.h>
+#if (LINUX_VERSION_CODE > VERSION(1,3,97))
+#include <scsi/scsi.h>
+#else
+#include <linux/scsi.h>
+#endif
+#endif
+
 #include "cardmgr.h"
 
 static const char *version =
-    "cardmgr.c 1.57 1995/07/05 16:06:28 (David Hinds)";
+    "cardmgr.c 1.80 1996/08/06 05:06:29 (David Hinds)";
 
 /*====================================================================*/
 
@@ -41,7 +53,7 @@ typedef struct socket_info_t {
     int fd;
     int state;
     card_info_t *card;
-    bind_info_t bind[MAX_FUNCTIONS];
+    bind_info_t *bind[MAX_FUNCTIONS];
     char *mtd[2*CISTPL_MAX_DEVICES];
 } socket_info_t;
 
@@ -73,11 +85,11 @@ struct mtd_ident_t *default_mtd = NULL;
 static int sockets;
 static struct socket_info_t socket[MAX_SOCKS];
 
-/* Default path for config file */
+/* Default path for config file, device scripts */
 #ifdef ETC
-static char *config = ETC "/config";
+static char *config = ETC;
 #else
-static char *config = "/etc/pcmcia/config";
+static char *config = "/etc/pcmcia";
 #endif
 
 /* Default path for pid file */
@@ -87,7 +99,7 @@ static char *pidfile = "/var/run/cardmgr.pid";
 static char *modpath = NULL;
 
 /* Default path for socket info table */
-static char *stabfile = "/etc/stab";
+static char *stabfile = "/var/run/stab";
 
 /* If set, don't generate beeps when cards are inserted */
 static int be_quiet = 0;
@@ -120,20 +132,70 @@ int lookup_dev(char *name)
 
 /*====================================================================*/
 
-int open_dev(dev_t dev)
+int open_dev(dev_t dev, int mode)
 {
     static char *fn;
     int fd;
 
     if ((fn = tmpnam(NULL)) == NULL)
 	return -1;
-    if (mknod(fn, (S_IFCHR|S_IREAD|S_IWRITE), dev) != 0)
+    if (mknod(fn, mode, dev) != 0)
 	return -1;
-    fd = open(fn, O_RDWR);
+    fd = open(fn, (mode & S_IWRITE) ? O_RDWR: O_RDONLY);
     unlink(fn);
     return fd;
 } /* open_dev */
 
+/*======================================================================
+
+    xlate_scsi_name() is a sort-of-hack used to deduce the minor
+    device numbers of SCSI devices, from the information available to
+    the low-level driver.
+    
+======================================================================*/
+
+#ifdef GET_SCSI_INFO
+static int xlate_scsi_name(bind_info_t *bind)
+{
+    int i, fd, mode, minor;
+    u_long arg[2], id1, id2;
+
+    id1 = strtol(bind->name+3, NULL, 16);
+    if ((bind->major == SCSI_DISK_MAJOR) ||
+	(bind->major == SCSI_CDROM_MAJOR))
+	mode = S_IREAD|S_IFBLK;
+    else
+	mode = S_IREAD|S_IFCHR;
+    
+    for (i = 0; i < 16; i++) {
+	minor = (bind->major == SCSI_DISK_MAJOR) ? (i<<4) : i;
+	fd = open_dev((bind->major<<8)+minor, mode);
+	if (fd < 0)
+	    continue;
+	if (ioctl(fd, SCSI_IOCTL_GET_IDLUN, arg) == 0) {
+	    id2 = (arg[0]&0x0f) + ((arg[0]>>4)&0xf0) +
+		((arg[0]>>8)&0xf00) + ((arg[0]>>12)&0xf000);
+	    if (id1 == id2) {
+		close(fd);
+		switch (bind->major) {
+		case SCSI_DISK_MAJOR:
+		case SCSI_GENERIC_MAJOR:
+		    sprintf(bind->name+2, "%c", 'a'+i); break;
+		case SCSI_CDROM_MAJOR:
+		    sprintf(bind->name, "scd%d", i); break;
+		case SCSI_TAPE_MAJOR:
+		    sprintf(bind->name+2, "%d", i); break;
+		}
+		bind->minor = minor;
+		return 0;
+	    }
+	}
+	close(fd);
+    }
+    return -1;
+} /* xlate_scsi_name */
+#endif
+
 /*====================================================================*/
 
 #define BEEP_TIME 150
@@ -160,9 +222,10 @@ void beep(unsigned int ms, unsigned int freq)
 
 void write_stab(void)
 {
-    int i, j;
+    int i, j, k;
     FILE *f;
     socket_info_t *s;
+    bind_info_t *bind;
 
     f = fopen(stabfile, "w");
     if (f == NULL) {
@@ -175,19 +238,26 @@ void write_stab(void)
     }
     for (i = 0; i < sockets; i++) {
 	s = &socket[i];
-	if (s->state & SOCKET_PRESENT) {
-	    if (s->card) {
-		fprintf(f, "Socket %d: %s\n", i, s->card->name);
-		for (j = 0; j < s->card->functions; j++)
-		    fprintf(f, "%2d\t%s\t%s\t%d\t%d\n",
-			    i, s->bind[j].dev_info, s->bind[j].name,
-			    s->bind[j].major, s->bind[j].minor);
-	    }
-	    else
-		fprintf(f, "Socket %d: unsupported card\n", i);
-	}
-	else
+	if (!(s->state & SOCKET_PRESENT))
 	    fprintf(f, "Socket %d: empty\n", i);
+	else if (!s->card)
+	    fprintf(f, "Socket %d: unsupported card\n", i);
+	else {
+	    fprintf(f, "Socket %d: %s\n", i, s->card->name);
+	    for (j = 0; j < s->card->functions; j++)
+		for (k = 0, bind = s->bind[j];
+		     bind != NULL;
+		     k++, bind = bind->next) {
+		    fprintf(f, "%d\t%s\t%s\t%d\t%s",
+			    i, s->card->device[j]->class,
+			    bind->dev_info, k, bind->name);
+		    if (bind->major)
+			fprintf(f, "\t%d\t%d\n",
+				bind->major, bind->minor);
+		    else
+			fputc('\n', f);
+		}
+	}
     }
     fflush(f);
     flock(fileno(f), LOCK_UN);
@@ -216,6 +286,8 @@ static int get_tuple(int ns, cisdata_t code, ds_ioctl_arg_t *arg)
     return 0;
 }
 
+/*====================================================================*/
+
 static card_info_t *lookup_card(int ns)
 {
     socket_info_t *s = &socket[ns];
@@ -320,14 +392,14 @@ static card_info_t *lookup_card(int ns)
 
 static void load_config(void)
 {
-    FILE *f;
-    f = fopen(config, "r");
-    if (f == NULL) {
-	syslog(LOG_INFO, "could not open %s: %m", config);
+    char fn[128];
+    sprintf(fn, "%s/config", config);
+    if (parse_configfile(fn) != 0)
 	exit(EXIT_FAILURE);
-    }
-    parse_configfile(f);
-    fclose(f);
+    if (root_device == NULL)
+	syslog(LOG_INFO, "no device drivers defined");
+    if (root_card == NULL)
+	syslog(LOG_INFO, "no cards defined");
 } /* load_config */
 
 /*====================================================================*/
@@ -366,8 +438,7 @@ static void free_config(void)
 	    free(dev->module[i]);
 	    if (dev->opts[i]) free(dev->opts[i]);
 	}
-	if (dev->start_cmd) free(dev->start_cmd);
-	if (dev->stop_cmd) free(dev->stop_cmd);
+	if (dev->class) free(dev->class);
 	free(dev);
     }
 
@@ -415,45 +486,25 @@ static int execute(char *msg, char *cmd)
 
 /*====================================================================*/
 
-static int execute_on_dev(char *msg, char *fmt,
-			  int sn, bind_info_t *bind)
+static int execute_on_dev(char *action, char *class, char *dev)
 {
-    char cmd[1024], *s;
-
-    s = cmd;
-    while (*fmt != '\0') {
-	if ((fmt[0] == '%') && (fmt[1] != '\0') && (fmt[2] == '%')) {
-	    fmt++;
-	    switch (*fmt) {
-	    case 'c':
-		strcpy(s, socket[sn].card->name);
-		break;
-	    case 'd':
-		strcpy(s, bind->name);
-		break;
-	    case 'm':
-		sprintf(s, "%d", bind->major);
-		break;
-	    case 'n':
-		sprintf(s, "%d", bind->minor);
-		break;
-	    case 's':
-		sprintf(s, "%d", sn);
-		break;
-	    default:
-		*s = '\0';
-	    }
-	    s += strlen(s);
-	    fmt += 2;
-	}
-	else
-	    *s++ = *fmt++;
-    }
-    *s = '\0';
+    char msg[128], cmd[512];
 
+    sprintf(msg, "%s cmd", action);
+    sprintf(cmd, "%s/%s %s %s", config, class, action, dev);
     return execute(msg, cmd);
+}
 
-} /* execute_on_dev */
+static int execute_on_all(char *cmd, char *class, int sn, int fn)
+{
+    socket_info_t *s = &socket[sn];
+    bind_info_t *bind;
+    int ret = 0;
+    for (bind = s->bind[fn]; bind != NULL; bind = bind->next)
+	if (bind->name[2] != '#')
+	    ret |= execute_on_dev(cmd, class, bind->name);
+    return ret;
+}
 
 /*====================================================================*/
 
@@ -467,6 +518,7 @@ static module_list_t *module_list = NULL;
 
 static int install_module(char *mod, char *opts)
 {
+    char path[128];
     char cmd[128];
     module_list_t *ml;
     
@@ -482,9 +534,14 @@ static int install_module(char *mod, char *opts)
     ml->usage++;
     if (ml->usage == 1) {
 	if (strchr(mod, '/') != NULL)
-	    sprintf(cmd, "/sbin/insmod %s/%s.o", modpath, mod);
+	    sprintf(path, "%s/%s.o", modpath, mod);
 	else
-	    sprintf(cmd, "/sbin/insmod %s/pcmcia/%s.o", modpath, mod);
+	    sprintf(path, "%s/pcmcia/%s.o", modpath, mod);
+	if (access(path, R_OK) != 0) {
+	    syslog(LOG_INFO, "module %s not available", path);
+	    return -1;
+	}
+	sprintf(cmd, "/sbin/insmod %s", path);
 	if (opts) {
 	    strcat(cmd, " ");
 	    strcat(cmd, opts);
@@ -606,7 +663,7 @@ static void do_insert(int sn)
     socket_info_t *s = &socket[sn];
     card_info_t *card;
     device_info_t **dev;
-    bind_info_t *bind;
+    bind_info_t *bind, **tail;
     int i, j, ret;
 
     /* Already identified? */
@@ -620,17 +677,14 @@ static void do_insert(int sn)
 	return;
     s->card = card;
 
-    /* Initialize socket bindings */
-    dev = card->device; bind = s->bind;
-    for (i = 0; i < card->functions; i++) {
-	strcpy((char *)bind[i].dev_info, (char *)dev[i]->dev_info);
-	bind[i].instance = NULL;
-	bind[i].name[0] = '\0';
-    }
-    write_stab();
+    dev = card->device;
 
     /* Set up MTD's */
-    bind_mtd(sn);
+    for (i = 0; i < card->functions; i++)
+	if (dev[i]->needs_mtd)
+	    break;
+    if (i < card->functions)
+	bind_mtd(sn);
     
     /* Install kernel modules */
     for (i = 0; i < card->functions; i++) {
@@ -640,16 +694,20 @@ static void do_insert(int sn)
 
     /* Bind drivers by their dev_info identifiers */
     for (i = 0; i < card->functions; i++) {
-	if (ioctl(s->fd, DS_BIND_REQUEST, &bind[i]) != 0) {
+	bind = calloc(1, sizeof(bind_info_t));
+	strcpy((char *)bind->dev_info, (char *)dev[i]->dev_info);
+	bind->function = card->dev_fn[i];
+	if (ioctl(s->fd, DS_BIND_REQUEST, bind) != 0) {
 	    syslog(LOG_INFO, "bind '%s' to socket %d failed: %m",
-		   (char *)bind[i].dev_info, sn);
+		   (char *)bind->dev_info, sn);
 	    if (errno != EBUSY) {
 		beep(BEEP_TIME, BEEP_ERR);
+		write_stab();
 		return;
 	    }
 	}
 	for (j = 0; j < 10; j++) {
-	    ret = ioctl(s->fd, DS_GET_DEVICE_INFO, &bind[i]);
+	    ret = ioctl(s->fd, DS_GET_DEVICE_INFO, bind);
 	    if ((ret == 0) || (errno != EAGAIN))
 		break;
 	    usleep(100000);
@@ -657,24 +715,32 @@ static void do_insert(int sn)
 	if (ret != 0) {
 	    syslog(LOG_INFO, "get dev info on socket %d failed: %m",
 		   sn);
+	    ioctl(s->fd, DS_UNBIND_REQUEST, bind);
 	    beep(BEEP_TIME, BEEP_ERR);
+	    write_stab();
 	    return;
 	}
-	if (strlen(bind[i].name) == 0) {
-	    syslog(LOG_INFO, "no dev info for socket %d", sn);
-	    beep(BEEP_TIME, BEEP_ERR);
-	    return;
+	tail = &s->bind[i];
+	while (ret == 0) {
+	    bind_info_t *old;
+#ifdef GET_SCSI_INFO
+	    if ((strlen(bind->name) > 3) &&
+		(bind->name[2] == '#'))
+		xlate_scsi_name(bind);
+#endif
+	    old = *tail = bind; tail = (bind_info_t **)&bind->next;
+	    bind = (bind_info_t *)malloc(sizeof(bind_info_t));
+	    memcpy(bind, old, sizeof(bind_info_t));
+	    ret = ioctl(s->fd, DS_GET_NEXT_DEVICE, bind);
 	}
+	*tail = NULL; free(bind);
 	write_stab();
     }
 
     /* Run "start" commands */
-    ret = 0;
-    for (i = 0; i < card->functions; i++) {
-	if (dev[i]->start_cmd)
-	    ret |= execute_on_dev("start cmd", dev[i]->start_cmd,
-				  sn, &bind[i]);
-    }
+    for (i = ret = 0; i < card->functions; i++)
+	if (dev[i]->class)
+	    ret |= execute_on_all("start", dev[i]->class, sn, i);
     if (ret)
 	beep(BEEP_TIME, BEEP_ERR);
     else
@@ -684,13 +750,38 @@ static void do_insert(int sn)
 
 /*====================================================================*/
 
-static int do_remove(int sn, int request)
+static int do_check(int sn)
+{
+    socket_info_t *s = &socket[sn];
+    card_info_t *card;
+    device_info_t **dev;
+    int i, ret;
+
+    card = s->card;
+    if (card == NULL)
+	return 0;
+    
+    /* Run "check" commands */
+    dev = card->device;
+    for (i = 0; i < card->functions; i++) {
+	if (dev[i]->class) {
+	    ret = execute_on_all("check", dev[i]->class, sn, i);
+	    if (ret != 0)
+		return CS_IN_USE;
+	}
+    }
+    return 0;
+}
+
+/*====================================================================*/
+
+static void do_remove(int sn)
 {
     socket_info_t *s = &socket[sn];
     card_info_t *card;
     device_info_t **dev;
     bind_info_t *bind;
-    int i, j, ret;
+    int i, j;
 
     card = s->card;
     if (card == NULL)
@@ -699,22 +790,24 @@ static int do_remove(int sn, int request)
     syslog(LOG_INFO, "shutting down socket %d", sn);
     
     /* Run "stop" commands */
-    dev = card->device; bind = s->bind;
+    dev = card->device;
     for (i = 0; i < card->functions; i++) {
-	if (dev[i]->stop_cmd && (bind[i].name[0] != '\0')) {
-	    ret = execute_on_dev("stop cmd", dev[i]->stop_cmd,
-				 sn, &bind[i]);
-	    if (request && (ret != 0))
-		goto fail;
+	if (dev[i]->class) {
+	    execute_on_all("stop", dev[i]->class, sn, i);
 	}
     }
 
     /* unbind driver instances */
     for (i = 0; i < card->functions; i++) {
-	if (bind[i].instance != NULL) {
-	    if (ioctl(s->fd, DS_UNBIND_REQUEST, &s->bind[i]) != 0)
+	if (s->bind[i]) {
+	    if (ioctl(s->fd, DS_UNBIND_REQUEST, s->bind[i]) != 0)
 		syslog(LOG_INFO, "unbind '%s' from socket %d failed: %m",
-		       (char *)bind[i].dev_info, sn);
+		       (char *)s->bind[i]->dev_info, sn);
+	    while (s->bind[i]) {
+		bind = s->bind[i];
+		s->bind[i] = bind->next;
+		free(bind);
+	    }
 	}
     }
 
@@ -731,16 +824,55 @@ done:
     beep(BEEP_TIME, BEEP_OK);
     s->card = NULL;
     write_stab();
-    return 0;
-fail:
-    beep(BEEP_TIME, BEEP_ERR);
-    write_stab();
-    return -EBUSY;
 }
 
 /*====================================================================*/
 
-void free_resources(void)
+static void do_suspend(int sn)
+{
+    socket_info_t *s = &socket[sn];
+    card_info_t *card;
+    device_info_t **dev;
+    int i, ret;
+    
+    card = s->card;
+    if (card == NULL)
+	return;
+    dev = card->device;
+    for (i = 0; i < card->functions; i++) {
+	if (dev[i]->class) {
+	    ret = execute_on_all("suspend", dev[i]->class, sn, i);
+	    if (ret != 0)
+		beep(BEEP_TIME, BEEP_ERR);
+	}
+    }
+}
+
+/*====================================================================*/
+
+static void do_resume(int sn)
+{
+    socket_info_t *s = &socket[sn];
+    card_info_t *card;
+    device_info_t **dev;
+    int i, ret;
+    
+    card = s->card;
+    if (card == NULL)
+	return;
+    dev = card->device;
+    for (i = 0; i < card->functions; i++) {
+	if (dev[i]->class) {
+	    ret = execute_on_all("resume", dev[i]->class, sn, i);
+	    if (ret != 0)
+		beep(BEEP_TIME, BEEP_ERR);
+	}
+    }
+}
+
+/*====================================================================*/
+
+static void free_resources(void)
 {
     adjust_list_t *al;
     int fd = socket[0].fd;
@@ -761,7 +893,7 @@ void free_resources(void)
 
 /*====================================================================*/
 
-void adjust_resources(void)
+static void adjust_resources(void)
 {
     adjust_list_t *al;
     int ret;
@@ -795,14 +927,25 @@ void adjust_resources(void)
     
 /*====================================================================*/
 
-void catch_signal(int sig)
+static int cleanup_files = 0;
+
+static void done(void)
+{
+    syslog(LOG_INFO, "exiting");
+    if (cleanup_files) {
+	unlink(pidfile);
+	unlink(stabfile);
+    }
+}
+
+static void catch_signal(int sig)
 {
     caught_signal = sig;
     if (signal(sig, catch_signal) == SIG_ERR)
 	syslog(LOG_INFO, "signal(%d): %m", sig);
 }
 
-void handle_signal(void)
+static void handle_signal(void)
 {
     int i;
     switch (caught_signal) {
@@ -810,14 +953,14 @@ void handle_signal(void)
     case SIGINT:
 	for (i = 0; i < sockets; i++)
 	    if (socket[i].state & SOCKET_PRESENT)
-		do_remove(i, 0);
+		do_remove(i);
 	free_resources();
 	exit(0);
 	break;
     case SIGHUP:
 	for (i = 0; i < sockets; i++)
 	    if (socket[i].state & SOCKET_PRESENT)
-		do_remove(i, 0);
+		do_remove(i);
 	free_resources();
 	free_config();
 	syslog(LOG_INFO, "re-loading config file");
@@ -832,16 +975,9 @@ void handle_signal(void)
     }
 }
 
-void done(void)
-{
-    syslog(LOG_INFO, "exiting");
-    unlink(pidfile);
-    unlink(stabfile);
-}
-
 /*====================================================================*/
 
-int init_sockets(void)
+static int init_sockets(void)
 {
     int major, fd, i;
     servinfo_t serv;
@@ -852,7 +988,7 @@ int init_sockets(void)
 	exit(EXIT_FAILURE);
     }
     for (i = 0; i < MAX_SOCKS; i++) {
-	fd = open_dev((major<<8)+i);
+	fd = open_dev((major<<8)+i, S_IFCHR|S_IREAD|S_IWRITE);
 	if (fd < 0) break;
 	socket[i].fd = fd;
 	socket[i].state = 0;
@@ -914,8 +1050,8 @@ void main(int argc, char *argv[])
 	}
     }
     if (errflg || (optind < argc)) {
-	fprintf(stderr, "usage: %s [-q] [-v] [-c config] [-m modpath] "
-		"[-p pidfile] [-s stabfile]\n", argv[0]);
+	fprintf(stderr, "usage: %s [-q] [-v] [-c configpath] "
+		"[-m modpath] [-p pidfile] [-s stabfile]\n", argv[0]);
 	exit(EXIT_FAILURE);
     }
 
@@ -956,6 +1092,9 @@ void main(int argc, char *argv[])
     
     load_config();
     
+    if (init_sockets() != 0)
+	exit(EXIT_FAILURE);
+
     f = fopen(pidfile, "w");
     if (f == NULL)
 	syslog(LOG_INFO, "could not open %s: %m", pidfile);
@@ -963,9 +1102,8 @@ void main(int argc, char *argv[])
 	fprintf(f, "%d\n", getpid());
 	fclose(f);
     }
-
-    if (init_sockets() != 0)
-	exit(EXIT_FAILURE);
+    /* If we've gotten this far, then clean up pid and stab on exit */
+    cleanup_files = 1;
 
     if (signal(SIGHUP, catch_signal) == SIG_ERR)
 	syslog(LOG_INFO, "signal(SIGHUP): %m");
@@ -1002,11 +1140,14 @@ void main(int argc, char *argv[])
 	    switch (event) {
 	    case CS_EVENT_CARD_REMOVAL:
 		socket[i].state &= ~(SOCKET_PRESENT | SOCKET_READY);
-		do_remove(i, 0);
+		do_remove(i);
 		break;
 	    case CS_EVENT_EJECTION_REQUEST:
-		socket[i].state &= ~(SOCKET_PRESENT | SOCKET_READY);
-		ret = do_remove(i, 1);
+		ret = do_check(i);
+		if (ret == 0) {
+		    socket[i].state &= ~(SOCKET_PRESENT | SOCKET_READY);
+		    do_remove(i);
+		}
 		write(socket[i].fd, &ret, 4);
 		break;
 	    case CS_EVENT_CARD_INSERTION:
@@ -1019,6 +1160,12 @@ void main(int argc, char *argv[])
 	    case CS_EVENT_RESET_PHYSICAL:
 		socket[i].state &= ~SOCKET_READY;
 		break;
+	    case CS_EVENT_PM_SUSPEND:
+		do_suspend(i);
+		break;
+	    case CS_EVENT_PM_RESUME:
+		do_resume(i);
+		break;
 	    }
 	    
 	}
