@@ -4,7 +4,7 @@
 
     Written by David Hinds, dhinds@allegro.stanford.edu
 
-    ftl_format.c 1.1 1995/11/06 17:40:14
+    ftl_format.c 1.2 1996/09/15 04:18:12
 
 ======================================================================*/
 
@@ -150,7 +150,7 @@ void main(int argc, char *argv[])
     
     errflg = 0;
     verbose = 0;
-    while ((optch = getopt(argc, argv, "q")) != -1) {
+    while ((optch = getopt(argc, argv, "v")) != -1) {
 	switch (optch) {
 	case 'v':
 	    verbose = 1; break;
