@@ -1,5 +1,5 @@
 /*
- * ciscode.h 1.5 1996/07/24 15:27:27 (David Hinds)
+ * ciscode.h $Revision: 1.5 $ $Date: 1996/07/24 15:27:27 $ (David Hinds)
  */
 
 #ifndef _LINUX_CISCODE_H
@@ -13,6 +13,7 @@
 #define MANFID_MEGAHERTZ	0x0102
 #define MANFID_SMC		0x0108
 #define MANFID_ADAPTEC		0x012f
+#define MANFID_QUATECH		0x0137
 #define MANFID_OSITECH		0x0140
 #define MANFID_LINKSYS		0x0143
 #define MANFID_ATT		0xffff
@@ -28,6 +29,7 @@
 #define PRODID_MEGAHERTZ_EM3288		0x0006
 #define PRODID_OSITECH_JACK_144		0x0001
 #define PRODID_OSITECH_JACK_288		0x0002
+#define PRODID_QUATECH_DUAL_RS232	0x0012
 #define PRODID_SMC_ETHER		0x0105
 
 #endif /* _LINUX_CISCODE_H */
