@@ -36,7 +36,7 @@
 #ifdef PCMCIA_DEBUG
 int pc_debug = PCMCIA_DEBUG;
 static const char *version =
-    "ds.c 1.56 1996/05/28 15:25:32 (David Hinds)";
+    "ds.c 1.57 1996/09/08 22:07:27 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -121,6 +121,18 @@ static struct file_operations ds_fops = {
     NULL		/* fsync */
 };
 
+#undef CONFIG_MODVERSIONS
+static struct symbol_table ds_symtab = {
+#include <linux/symtab_begin.h>
+#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
+#undef X
+#define X(sym) { (void *)&sym, SYMBOL_NAME_STR(sym) }
+#endif
+    X(register_pcmcia_driver),
+    X(unregister_pcmcia_driver),
+#include <linux/symtab_end.h>
+};
+
 /*====================================================================*/
 
 static void cs_error(int func, int ret)
@@ -870,6 +882,7 @@ int init_ds(void)
 	       "Driver Services\n");
     else
 	major_dev = i;
+    register_symtab(&ds_symtab);
     
     return 0;
 }
