@@ -5,7 +5,8 @@
 
     Written by Shingo Fujimoto, shingo@flab.fujitsu.co.jp
 
-    TDK LAK-CD021 support added by Nobuhiro Katayama, kata-n@po.iijnet.or.jp
+    TDK LAK-CD021 and CONTEC C-NET(PC)C support added by 
+    Nobuhiro Katayama, kata-n@po.iijnet.or.jp
 
     Network code is based on the "FMV-18x driver" by Yutaka TAMIYA
 
@@ -70,12 +71,18 @@ static fmvj18x_debug = 2;
 /* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
 static u_long irq_mask = 0xdeb8;
 
+/* SRAM configuration */
+/* 0:4KB*2 TX buffer   else:8KB*2 TX buffer */
+static int sram_config = 0;
+
 /*====================================================================*/
 /* 
    driver version infomation 
  */
+#ifdef PCMCIA_DEBUG
 static char *version =
  "fmvj18x_cs.c,v 1.9 1996/08/06 03:13:53 root Exp";
+#endif
 
 /*====================================================================*/
 /*
@@ -113,7 +120,7 @@ static dev_link_t *dev_list = NULL;
 /*
     card type
  */
-typedef enum { MBH10302, MBH10304, TDK} cardtype_t;
+typedef enum { MBH10302, MBH10304, TDK, CONTEC } cardtype_t;
 
 /*
     driver specific data structure
@@ -191,7 +198,7 @@ typedef struct local_info_t {
 
 #define F_BUF_EMP            0x40 /* receive buffer is empty */
 
-#define F_SKP_PKT            0x04 /* drop packet in buffer */
+#define F_SKP_PKT            0x05 /* drop packet in buffer */
 
 /* default bitmaps */
 #define D_TX_INTR  ( ENA_TMT_OK )
@@ -206,7 +213,9 @@ typedef struct local_info_t {
 #define ID_MATCHED           0x02 /* (RX_MODE) */
 #define RECV_ALL             0x03 /* (RX_MODE) */
 #define CONFIG0_DFL          0x5a /* 16bit bus, 4K x 2 Tx queues */
+#define CONFIG0_DFL_1        0x5e /* 16bit bus, 8K x 2 Tx queues */
 #define CONFIG0_RST          0xda /* Data Link Controler off (CONFIG_0) */
+#define CONFIG0_RST_1        0xde /* Data Link Controler off (CONFIG_0) */
 #define BANK_0               0xa0 /* bank 0 (CONFIG_1) */
 #define BANK_1               0xa4 /* bank 1 (CONFIG_1) */
 #define BANK_2               0xa8 /* bank 2 (CONFIG_1) */
@@ -415,8 +424,12 @@ static void fmvj18x_config(dev_link_t *link)
 	if( tuple.TupleData[0] == 0x05 && tuple.TupleData[1] == 0x01 ) {
 	    /* Manufacture is TDK */
 	    cardtype = TDK;
+	    link->conf.ConfigIndex = 0x04;
+	} else if( tuple.TupleData[0] == 0x01 && tuple.TupleData[1] == 0xc0 ) {
+	    /* Manufacture is CONTEC */
+	    cardtype = CONTEC;
 	    link->conf.ConfigIndex = 1;
-	}else {
+	} else {
 	    /* FMV-J182(MBH10304) have MACID on CIS */
 	    cardtype = MBH10304;
 	    link->conf.ConfigIndex = 0x60;
@@ -458,12 +471,16 @@ static void fmvj18x_config(dev_link_t *link)
     /* Power On chip and select bank 0 */
     outb(BANK_0, ioaddr + CONFIG_1);
     /* Reset controler */
-    outb(CONFIG0_RST, ioaddr + CONFIG_0);
+    if( sram_config == 0 ) 
+	outb(CONFIG0_RST, ioaddr + CONFIG_0);
+    else
+	outb(CONFIG0_RST_1, ioaddr + CONFIG_0);
     
     /* Set hardware address */
     switch (cardtype) {
     case MBH10304:
     case TDK:
+    case CONTEC:
 	do {
 	    u_char *node_id;
 	    
@@ -486,7 +503,11 @@ static void fmvj18x_config(dev_link_t *link)
 		    if (ret != CS_SUCCESS) goto end;
 		}
 		node_id = &(tuple.TupleData[2]);
-		card_name = "TDK LAK-CD021";
+		if( cardtype == TDK ) {
+		    card_name = "TDK LAK-CD021";
+		} else {
+		    card_name = "C-NET(PC)C";
+		}
 	    }
 	    /* Read MACID from CIS */
 	    for (i = 0; i < 6; i++)
@@ -507,8 +528,9 @@ static void fmvj18x_config(dev_link_t *link)
 
     ((struct local_info_t *)dev->priv)->cardtype = cardtype ;
     /* print current configuration */
-    printk(KERN_INFO "%s: %s, port %#3lx, irq %d, hw_addr ", 
-	   dev->name, card_name, dev->base_addr, dev->irq);
+    printk(KERN_INFO "%s: %s, sram %s, port %#3lx, irq %d, hw_addr ", 
+	   dev->name, card_name, sram_config == 0 ? "4K TX*2" : "8K TX*2", 
+	   dev->base_addr, dev->irq);
     for (i = 0; i < 6; i++)
         printk(" %02X", dev->dev_addr[i]);
     printk("\n");
@@ -825,7 +847,7 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
 	outsw(ioaddr + DATAPORT, buf, (length + 1) >> 1);
 
 	lp->tx_queue++;
-	lp->tx_queue_len += length + 2;
+	lp->tx_queue_len += ((length+3) & ~1);
 
 	if (lp->tx_started == 0) {
 	    /* If the Tx is idle, always trigger a transmit. */
@@ -836,9 +858,18 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
 	    dev->trans_start = jiffies;
 	    lp->tx_started = 1;
 	    dev->tbusy = 0;
-	} else if (lp->tx_queue_len < (4096 - (ETH_FRAME_LEN +2)) )
-	    /* Yes, there is room for one more packet. */
-	    dev->tbusy = 0;
+	} else {
+	    if( sram_config == 0 ) {
+		if (lp->tx_queue_len < (4096 - (ETH_FRAME_LEN +2)) )
+		    /* Yes, there is room for one more packet. */
+		    dev->tbusy = 0;
+	    } else {
+		if (lp->tx_queue_len < (8192 - (ETH_FRAME_LEN +2)) && 
+						lp->tx_queue < 127 )
+		    /* Yes, there is room for one more packet. */
+		    dev->tbusy = 0;
+	    }
+	}
 
 	/* Re-enable interrupts */
 	outb(D_TX_INTR, ioaddr + TX_INTR);
@@ -864,7 +895,10 @@ static void fjn_reset(struct device *dev)
     /* Power On chip and select bank 0 */
     outb(BANK_0, ioaddr + CONFIG_1);
     /* Reset buffers */
-    outb(CONFIG0_RST, ioaddr + CONFIG_0);
+    if( sram_config == 0 ) 
+	outb(CONFIG0_RST, ioaddr + CONFIG_0);
+    else
+	outb(CONFIG0_RST_1, ioaddr + CONFIG_0);
 
     /* Set Tx modes */
     outb(D_TX_MODE, ioaddr + TX_MODE);
@@ -902,10 +936,13 @@ static void fjn_reset(struct device *dev)
     outb(0x00, ioaddr + BMPR13);
 
     /* reset Skip packet reg. */
-    outb(0x00, ioaddr + RX_SKIP);
+    outb(0x01, ioaddr + RX_SKIP);
 
     /* Enable Tx and Rx */
-    outb(CONFIG0_DFL, ioaddr + CONFIG_0);
+    if( sram_config == 0 )
+	outb(CONFIG0_DFL, ioaddr + CONFIG_0);
+    else
+	outb(CONFIG0_DFL_1, ioaddr + CONFIG_0);
 
     /* Init receive pointer ? */
     inw(ioaddr + DATAPORT);
@@ -915,14 +952,16 @@ static void fjn_reset(struct device *dev)
     outb(0xff, ioaddr + TX_STATUS);
     outb(0xff, ioaddr + RX_STATUS);
 
-    outb(INTR_OFF, ioaddr + LAN_CTRL);
+    if( lp->cardtype != TDK ) 
+    		outb(INTR_OFF, ioaddr + LAN_CTRL);
 
     /* Turn on Rx interrupts */
     outb(D_TX_INTR, ioaddr + TX_INTR);
     outb(D_RX_INTR, ioaddr + RX_INTR);
 
     /* Turn on interrupts from LAN card controler */
-    outb(INTR_ON, ioaddr + LAN_CTRL);
+    if( lp->cardtype != TDK ) 
+		outb(INTR_ON, ioaddr + LAN_CTRL);
 } /* fjn_reset */
 
 /*====================================================================*/
@@ -1001,12 +1040,13 @@ static void fjn_rx(struct device *dev)
     /* If any worth-while packets have been received, dev_rint()
 	   has done a mark_bh(NET_BH) for us and will work on them
 	   when we get to the bottom-half routine. */
+/*
     if( lp->cardtype != TDK ) {
 	int i;
 	for (i = 0; i < 20; i++) {
 	    if ((inb(ioaddr + RX_MODE) & F_BUF_EMP) == F_BUF_EMP)
 		break;
-	    (void)inw(ioaddr + DATAPORT);  /* dummy status read */
+	    (void)inw(ioaddr + DATAPORT);  /+ dummy status read +/
 	    outb(F_SKP_PKT, ioaddr + RX_SKIP);
 	}
 
@@ -1014,6 +1054,7 @@ static void fjn_rx(struct device *dev)
 	    printk(KERN_DEBUG "%s: Exint Rx packet with mode %02x after"
 		   " %d ticks.\n", dev->name, inb(ioaddr + RX_MODE), i);
     }
+*/
 
     return;
 } /* fjn_rx */
@@ -1059,6 +1100,7 @@ static int fjn_open(struct device *dev)
 static int fjn_close(struct device *dev)
 {
     int ioaddr = dev->base_addr;
+    struct local_info_t *lp = (struct local_info_t *)dev->priv;
     dev_link_t *link;
 
     if (fmvj18x_debug > 4)
@@ -1078,7 +1120,10 @@ static int fjn_close(struct device *dev)
     dev->start = 0;
 
     /* Set configuration register 0 to disable Tx and Rx. */
-    outb(CONFIG0_RST ,ioaddr + CONFIG_0);
+    if( sram_config == 0 ) 
+	outb(CONFIG0_RST ,ioaddr + CONFIG_0);
+    else
+	outb(CONFIG0_RST_1 ,ioaddr + CONFIG_0);
 
     /* Update the statistics -- ToDo. */
 
@@ -1086,7 +1131,8 @@ static int fjn_close(struct device *dev)
     outb(CHIP_OFF ,ioaddr + CONFIG_1);
 
     /* Set the ethernet adaptor disable IRQ */
-    outb(INTR_OFF, ioaddr + LAN_CTRL);
+    if( lp->cardtype != TDK ) 
+    		outb(INTR_OFF, ioaddr + LAN_CTRL);
 
     link->open--;
     dev->start = 0;
