@@ -59,7 +59,7 @@ static int major_dev = 0;
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"ftl_cs.c 1.18 1996/05/28 15:16:02 (David Hinds)";
+"ftl_cs.c 1.20 1996/09/21 05:07:21 (David Hinds)";
 #endif
 
 static int shuffle_freq = 50;
@@ -542,8 +542,7 @@ static int scan_partition(partition_t *part)
     
     /* Set up virtual page map */
     blocks = header.FormattedSize >> header.BlockSize;
-    part->VirtualBlockMap =
-	kmalloc(blocks * sizeof(u_long), GFP_KERNEL);
+    part->VirtualBlockMap = vmalloc(blocks * sizeof(u_long));
     memset(part->VirtualBlockMap, 0xff, blocks * sizeof(u_long));
     part->BlocksPerUnit = (1 << header.EraseUnitSize) >> header.BlockSize;
     req.Count = part->BlocksPerUnit * sizeof(u_long);
@@ -1039,8 +1038,10 @@ static int ftl_open(struct inode *inode, struct file *file)
 	    return -ENODEV;
 	if (scan_partition(partition) == 0) {
 	    partition->state = FTL_FORMATTED;
+#ifdef PCMCIA_DEBUG
 	    printk(KERN_INFO "ftl_cs: opening %ld kb FTL partition\n",
 		   partition->header.FormattedSize >> 10);
+#endif
 	}
 	else {
 	    CardServices(CloseMemory, partition->handle);
@@ -1071,7 +1072,7 @@ static void ftl_close(struct inode *inode, struct file *file)
 #endif
 
     /* Flush all writes */
-    sync_dev(inode->i_rdev);
+    fsync_dev(inode->i_rdev);
 
     link = dev_table[DEVICE_NR(minor)];
     dev = (ftl_dev_t *)link->priv;
@@ -1091,7 +1092,7 @@ static void ftl_close(struct inode *inode, struct file *file)
 	CardServices(CloseMemory, part->handle);
 	part->handle = NULL;
 	if (part->VirtualBlockMap) {
-	    kfree(part->VirtualBlockMap);
+	    vfree(part->VirtualBlockMap);
 	    part->VirtualBlockMap = NULL;
 	}
 	if (part->VirtualPageMap) {
