@@ -43,7 +43,7 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static const char *version =
-"i82365.c 1.99 1996/08/15 03:16:51 (David Hinds)";
+"i82365.c 1.102 1996/10/01 14:25:39 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -276,37 +276,29 @@ static int to_ns(int cycles)
 static int identify(int sock)
 {
     u_char val;
-    int type;
+    int type = -1;
     
-    val = i365_get(sock, I365_IDENT);
+    if ((val = i365_get(sock, I365_IDENT)) & 0x70)
+	return -1;
     switch (val) {
     case 0x82:
 	type = IS_I82365A; break;
     case 0x83:
 	type = IS_I82365B; break;
     case 0x84:
-	type = IS_VLSI; break;
+	type = IS_I82365DF; break;
     case 0x88: case 0x89: case 0x8a:
 	type = IS_IBM; break;
-    case 0x8b: case 0x8c:
-	/* Special: fall into Vadem test */
-	type = IS_VG468; break;
-    default:
-	return -1;
     }
     
     /* Check for Vadem VG-468 chips */
-    if ((type == IS_I82365B) || (type == IS_VG468)) {
-	outb_p(0x0e, I365_PORT(sock));
-	outb_p(0x37, I365_PORT(sock));
-	i365_bset(sock, VG468_MISC, VG468_MISC_VADEMREV);
-	val = i365_get(sock, I365_IDENT);
-	if (val & I365_IDENT_VADEM) {
-	    type = ((val & 7) >= 4) ? IS_VG469 : IS_VG468;
-	    i365_bclr(sock, VG468_MISC, VG468_MISC_VADEMREV);
-	}
-	else if (type == IS_VG468)
-	    return -1;
+    outb_p(0x0e, I365_PORT(sock));
+    outb_p(0x37, I365_PORT(sock));
+    i365_bset(sock, VG468_MISC, VG468_MISC_VADEMREV);
+    val = i365_get(sock, I365_IDENT);
+    if (val & I365_IDENT_VADEM) {
+	i365_bclr(sock, VG468_MISC, VG468_MISC_VADEMREV);
+	type = ((val & 7) >= 4) ? IS_VG469 : IS_VG468;
     }
 
     /* Check for Ricoh chips */
@@ -506,23 +498,22 @@ static int i365_init(void)
     }
     
     type = identify(0);
-    if (type == IS_VLSI) {
+    if ((type == IS_I82365DF) && (identify(1) != type)) {
 	for (i = 0; i < 4; i++) {
 	    if (i == ignore) continue;
 	    j = ((i & 1) << 3) + (i << 1);
-	    if ((identify(j) == type) && !is_active(j)) {
+	    if ((identify(j) == IS_I82365DF) && !is_active(j)) {
 		socket_table[sockets].psock = j;
-		socket_table[sockets].type = type;
+		socket_table[sockets].type = IS_VLSI;
 		sockets++;
 	    }
 	}
-	add_pcic(type, sockets);
-    }
-    else
+	add_pcic(IS_VLSI, sockets);
+    } else {
 	for (i = 0; i < (extra_sockets ? 8 : 4); i += 2) {
 	    id = identify(i);
 	    if (id < 0) continue;
-
+	    
 	    if (is_pci && (id == IS_PD672X))
 		type = IS_PD6729;
 	    else
@@ -556,7 +547,8 @@ static int i365_init(void)
 	    if (ns != 0)
 		add_pcic(type, ns);
 	}
-    
+    }
+	
     if (sockets == 0) {
 	printk("not found.\n");
 	return -ENODEV;
