@@ -11,13 +11,13 @@
     
 ======================================================================*/
 
-#include <linux/config.h>
+#include "config.h"
+#include "k_compat.h"
 
 /* #define PCMCIA_DEBUG 1 */
 
 #ifdef MODULE
-#include <linux/module.h>
-#include <linux/version.h>
+#define init_iflash2X_mtd init_module
 #endif
 
 #include <linux/kernel.h>
@@ -40,18 +40,21 @@
 #include "bulkmem.h"
 #include "cistpl.h"
 #include "ds.h"
+#include "mem_op.h"
 #include "iflash.h"
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"iflash2+_mtd.c 1.10 1995/07/31 04:04:35 (David Hinds)\n";
+"iflash2+_mtd.c 1.23 1996/05/28 15:14:24 (David Hinds)";
 #endif
 
-static int vpp_timeout_period	= 1000;	/* in ticks */
-static int vpp_settle		= 10;	/* in ticks */
-static int erase_timeout	= 100;	/* in ms */
-static int erase_limit		= 1000;	/* in ticks */
+/*====================================================================*/
+
+static int vpp_timeout_period	= HZ;		/* in ticks */
+static int vpp_settle		= HZ/10;	/* in ticks */
+static int erase_timeout	= 100;		/* in ms */
+static int erase_limit		= 10*HZ;	/* in ticks */
 
 static int retry_limit		= 4;	/* max retries */
 
@@ -106,17 +109,6 @@ static dev_link_t *dev_list = NULL;
 
 /*====================================================================*/
 
-static void kmsg(char *fmt, ...)
-{
-    char buf[256];
-    va_list args;
-    
-    va_start(args, fmt);
-    vsprintf(buf, fmt, args);
-    va_end(args);
-    printk("%s: %s\n", (char *)dev_info, buf);
-}
-
 static void cs_error(int func, int ret)
 {
     CardServices(ReportError, dev_info, (void *)func, (void *)ret);
@@ -145,75 +137,73 @@ static inline u_long uticks(void)
 static void log_esr(char *s, volatile u_short *esr)
 {
     u_short CSR, GSR, BSR;
-    
-    *esr = IF_READ_CSR;
-    CSR = *esr;
-    *esr = IF_READ_ESR;
-    BSR = esr[2];
-    GSR = esr[4];
+
+    writew(IF_READ_CSR, esr);
+    CSR = readw(esr);
+    writew(IF_READ_ESR, esr);
+    BSR = readw(esr+2);
+    GSR = readw(esr+4);
     if (s)
-	printk("%s: CSR = 0x%04x, BSR = 0x%04x, GSR = 0x%04x\n",
-	       s, CSR, BSR, GSR);
-    else
-	printk("CSR = 0x%04x, BSR = 0x%04x, GSR = 0x%04x\n",
-	       CSR, BSR, GSR);
+	printk("%s: ", s);
+    printk("CSR = 0x%04x, BSR = 0x%04x, GSR = 0x%04x\n",
+	   CSR, BSR, GSR);
 }
 
 static void abort_cmd(volatile u_short *esr)
 {
     u_short i;
-    
-    *esr = IF_ABORT;
-    *esr = IF_READ_ESR;
+
+    writew(IF_ABORT, esr);
+    writew(IF_READ_ESR, esr);
     for (i = 0; i < max_tries; i++)
-	if ((esr[4] & GSR_SLEEP) == GSR_SLEEP) break;
+	if ((readw(esr+4) & GSR_SLEEP) == GSR_SLEEP) break;
     if (i == max_tries)
-	kmsg("abort cmd failed!");
-    *esr = IF_READ_ARRAY;
-    *esr = IF_CLEAR_CSR;
+	printk(KERN_NOTICE "iflash2+_mtd: abort cmd failed!\n");
+    writew(IF_READ_ARRAY, esr);
+    writew(IF_CLEAR_CSR, esr);
 }
 
 static int set_rdy_mode(volatile u_short *esr, u_short mode)
 {
     u_short i;
-    
-    *esr = IF_READ_ESR;
+
+    writew(IF_READ_ESR, esr);
     for (i = 0; i < max_tries; i++)
-	if (!(esr[2] & BSR_QUEUE_FULL)) break;
+	if (!(readw(esr+2) & BSR_QUEUE_FULL)) break;
     if (i == max_tries) {
-	kmsg("set_rdy_mode timed out!");
+	printk(KERN_NOTICE "iflash2+_mtd: set_rdy_mode timed out!\n");
 	log_esr(NULL, esr);
 	return CS_GENERAL_FAILURE;
     }
-    *esr = IF_RDY_MODE;
-    *esr = mode;
-    *esr = IF_READ_ESR;
+    writew(IF_RDY_MODE, esr);
+    writew(mode, esr);
+    writew(IF_READ_ESR, esr);
     for (i = 0; i < max_tries; i++) {
-	if ((esr[4] & GSR_WR_READY) == GSR_WR_READY) break;
+	if ((readw(esr+4) & GSR_WR_READY) == GSR_WR_READY) break;
     }
     if (i == max_tries) {
-	kmsg("set_rdy_mode timed out!");
+	printk(KERN_NOTICE "iflash2+_mtd: set_rdy_mode timed out!\n");
 	log_esr(NULL, esr);
 	return CS_GENERAL_FAILURE;
     }
-    if (esr[4] & GSR_OP_ERR)
+    if (readw(esr+4) & GSR_OP_ERR)
 	return CS_GENERAL_FAILURE;
     return CS_SUCCESS;
 }
 
 static int check_write(struct wait_queue **queue, volatile u_short *esr)
 {
-    *esr = IF_READ_ESR;
-    if ((esr[2] & BSR_READY) != BSR_READY) {
+    writew(IF_READ_ESR, esr);
+    if ((readw(esr+2) & BSR_READY) != BSR_READY) {
 	current->timeout = jiffies + 2;
 	interruptible_sleep_on(queue);
-	if ((esr[2] & BSR_READY) != BSR_READY) {
-	    kmsg("check_write: timed out!");
+	if ((readw(esr+2) & BSR_READY) != BSR_READY) {
+	    printk(KERN_NOTICE "iflash2+_mtd: check_write: timed out!\n");
 	    log_esr(NULL, esr);
 	    return CS_GENERAL_FAILURE;
 	}
     }
-    if (esr[2] & BSR_FAILED) {
+    if (readw(esr+2) & BSR_FAILED) {
 	log_esr("write error", esr);
 	return CS_WRITE_FAILURE;
     }
@@ -225,13 +215,13 @@ static int page_setup(struct wait_queue **queue, volatile u_short *esr,
 		      volatile u_short *address,  u_short count)
 {
     u_short nw;
-    
-    *esr = IF_READ_ESR;
-    if ((esr[4] & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) {
+
+    writew(IF_READ_ESR, esr);
+    if ((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) {
 	current->timeout = jiffies + 2;
 	interruptible_sleep_on(queue);
-	if ((esr[4] & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) {
-	    kmsg("page_setup timed out");
+	if ((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) {
+	    printk(KERN_NOTICE "iflash2+_mtd: page_setup timed out\n");
 	    log_esr(NULL, esr);
 	    return CS_GENERAL_FAILURE;
 	}
@@ -239,19 +229,19 @@ static int page_setup(struct wait_queue **queue, volatile u_short *esr,
     nw = count >> 1;
     if (nw == 0) {
 	/* Special case of single byte write */
-	*(volatile u_char *)address = LOW(IF_SINGLE_LOAD);
+	writeb(LOW(IF_SINGLE_LOAD), address);
     }
     else {
-	*address = IF_SEQ_LOAD;
+	writew(IF_SEQ_LOAD, address);
 	/* Tricky: split count into low bytes and high bytes */
 	nw = (count-nw-1) | ((nw-1) << 8);
 	if ((u_long)address & 2) {
-	    *address = 0;
-	    *address = nw;
+	    writew(0, address);
+	    writew(nw, address);
 	}
 	else {
-	    *address = nw;
-	    *address = 0;
+	    writew(nw, address);
+	    writew(0, address);
 	}
     }
     return CS_SUCCESS;
@@ -263,31 +253,31 @@ static int page_write(volatile u_short *esr, volatile u_short *address,
     u_short nw;
     u_short i;
 
-    *esr = IF_READ_ESR;
+    writew(IF_READ_ESR, esr);
     for (i = 0; i < max_tries; i++)
-	if (!(esr[2] & BSR_QUEUE_FULL)) break;
+	if (!(readw(esr+2) & BSR_QUEUE_FULL)) break;
     if (i == max_tries) {
-	kmsg("page_write timed out");
+	printk(KERN_NOTICE "iflash2+_mtd: page_write timed out\n");
 	log_esr(NULL, esr);
 	return CS_GENERAL_FAILURE;
     }
     nw = count >> 1;
     if (nw == 0) {
-	*(volatile u_char *)address = LOW(IF_PAGE_WRITE);
-	*(volatile u_char *)address = 0;
-	*(volatile u_char *)address = 0;
+	writeb(LOW(IF_PAGE_WRITE), address);
+	writeb(0, address);
+	writeb(0, address);
     }
     else {
-	*address = IF_PAGE_WRITE;
+	writew(IF_PAGE_WRITE, address);
 	/* Tricky: split count into low bytes and high bytes */
 	nw = (count-nw-1) | ((nw-1) << 8);
 	if ((u_long)address & 2) {
-	    *address = 0;
-	    *address = nw;
+	    writew(0, address);
+	    writew(nw, address);
 	}
 	else {
-	    *address = nw;
-	    *address = 0;
+	    writew(nw, address);
+	    writew(0, address);
 	}
     }
     return CS_SUCCESS;
@@ -295,15 +285,15 @@ static int page_write(volatile u_short *esr, volatile u_short *address,
 
 static void block_erase(volatile u_short *address)
 {
-    *address = IF_BLOCK_ERASE;
-    *address = IF_CONFIRM;
+    writew(IF_BLOCK_ERASE, address);
+    writew(IF_CONFIRM, address);
 }
 
 static int check_erase(volatile u_short *address)
 {
     u_short CSR;
-    *address = IF_READ_CSR;
-    CSR = *address;
+    writew(IF_READ_CSR, address);
+    CSR = readw(address);
     if ((CSR & CSR_WR_READY) != CSR_WR_READY)
 	return CS_BUSY;
     else if (CSR & (CSR_ERA_ERR | CSR_VPP_LOW | CSR_WR_ERR)) {
@@ -320,18 +310,18 @@ static int suspend_erase(volatile u_short *esr)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("suspending erase...");
+	printk(KERN_DEBUG "iflash2+_mtd: suspending erase...\n");
 #endif
-    *esr = IF_ERASE_SUSPEND;
-    *esr = IF_READ_ESR;
+    writew(IF_ERASE_SUSPEND, esr);
+    writew(IF_READ_ESR, esr);
     for (i = 0; i < max_tries; i++)
-	if ((esr[4] & GSR_WR_READY) == GSR_WR_READY) break;
+	if ((readw(esr+4) & GSR_WR_READY) == GSR_WR_READY) break;
     if (i == max_tries) {
-	kmsg("suspend_erase timed out");
+	printk(KERN_NOTICE "iflash2+_mtd: suspend_erase timed out\n");
 	log_esr(NULL, esr);
 	return CS_GENERAL_FAILURE;
     }
-    *esr = IF_READ_ARRAY;
+    writew(IF_READ_ARRAY, esr);
     return CS_SUCCESS;
 }
 
@@ -339,18 +329,18 @@ static void resume_erase(volatile u_short *esr)
 {
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("resuming erase...");
+	printk(KERN_DEBUG "iflash2+_mtd: resuming erase...\n");
 #endif
-    *esr = IF_READ_ESR;
+    writew(IF_READ_ESR, esr);
     /* Only give resume signal if the erase is really suspended */
-    if (esr[4] & GSR_OP_SUSPEND)
-	*esr = IF_CONFIRM;
+    if (readw(esr+4) & GSR_OP_SUSPEND)
+	writew(IF_CONFIRM, esr);
 }
 
 static void reset_block(volatile u_short *esr)
 {
-    *esr = IF_READ_ARRAY;
-    *esr = IF_CLEAR_CSR;
+    writew(IF_READ_ARRAY, esr);
+    writew(IF_CLEAR_CSR, esr);
 }
 
 /*====================================================================*/
@@ -363,7 +353,7 @@ static void set_global_lock(window_handle_t win,
     mod.AccessSpeed = 250;
     mod.CardOffset = 0x4000;
     MTDHelperEntry(MTDModifyWindow, win, &mod);
-    base[CISREG_WP] = (set) ? WP_BLKEN : 0;
+    writeb((set) ? WP_BLKEN : 0, base+CISREG_WP);
 }
 
 /*======================================================================
@@ -398,7 +388,7 @@ static int vpp_setup(dev_link_t *link, mtd_request_t *req)
 	    else {
 #ifdef PCMCIA_DEBUG
 		if (pc_debug > 1)
-		    kmsg("raising Vpp...");
+		    printk(KERN_DEBUG "iflash2+_mtd: raising Vpp...\n");
 #endif
 		dev->vpp_start = jiffies;
 		vpp_req.Vpp1 = vpp_req.Vpp2 = 120;
@@ -423,7 +413,7 @@ static void vpp_off(u_long arg)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("lowering Vpp...");
+	printk(KERN_DEBUG "iflash2+_mtd: lowering Vpp...\n");
 #endif
     dev = (flash_dev_t *)link->priv;
     dev->vpp_timeout.expires = 0;
@@ -437,29 +427,11 @@ static void vpp_shutdown(dev_link_t *link)
     dev = (flash_dev_t *)link->priv;
     dev->vpp_usage--;
     if (dev->vpp_usage == 0) {
-	dev->vpp_timeout.expires = vpp_timeout_period;
+	dev->vpp_timeout.expires = RUN_AT(vpp_timeout_period);
 	add_timer(&dev->vpp_timeout);
     }
 }
 
-/*======================================================================
-
-    If we're linked into the kernel, this gets called from init.c.
-    
-======================================================================*/
-
-#ifndef MODULE
-u_long flash_init(u_long kmem_start, u_long kmem_end)
-{
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(version);
-#endif
-    register_pcmcia_driver(&dev_info, &flash_attach, &flash_detach);
-    return kmem_start;
-}
-#endif
-
 /*======================================================================
 
     flash_attach() creates an "instance" of the driver, allocating
@@ -477,7 +449,7 @@ static dev_link_t *flash_attach(void)
     
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("flash_attach()");
+	printk(KERN_DEBUG "iflash2+_mtd: flash_attach()\n");
 #endif
 
     /* Create new memory card device */
@@ -531,7 +503,7 @@ static void flash_detach(dev_link_t *link)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("flash_detach(0x%p)", link);
+	printk(KERN_DEBUG "iflash2+_mtd: flash_detach(0x%p)\n", link);
 #endif
     
     /* Locate device structure */
@@ -573,7 +545,9 @@ static void flash_detach(dev_link_t *link)
 
 static void printk_size(u_long sz)
 {
-    if (sz & 0xfffff)
+    if (sz & 0x3ff)
+	printk("%ld bytes", sz);
+    else if (sz & 0x0fffff)
 	printk("%ld kb", sz >> 10);
     else
 	printk("%ld mb", sz >> 20);
@@ -589,7 +563,7 @@ static void flash_config(dev_link_t *link)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("flash_config(0x%p)", link);
+	printk(KERN_DEBUG "iflash2+_mtd: flash_config(0x%p)\n", link);
 #endif
 
     /* Allocate a 4K memory window */
@@ -637,7 +611,7 @@ static void flash_config(dev_link_t *link)
 		kfree_s(dev->flash[i], sizeof(struct flash_region_t));
 		break;
 	    }
-	    printk("iflash2+_mtd: %s at 0x%lx, ",
+	    printk(KERN_INFO "iflash2+_mtd: %s at 0x%lx, ",
 		   attr ? "attr" : "common", region.CardOffset);
 	    printk_size(region.RegionSize);
 	    printk(", %ld ns\n", region.AccessSpeed);
@@ -671,7 +645,7 @@ static void flash_release(u_long arg)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("flash_release(0x%p)", link);
+	printk(KERN_DEBUG "iflash2+_mtd: flash_release(0x%p)\n", link);
 #endif
 
     link->state &= ~DEV_CONFIG;
@@ -721,9 +695,9 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("flash_read(0x%p, 0x%ld, 0x%p, 0x%lx, 0x%lx)",
-	     link, req->MediaID, buf, req->SrcCardOffset,
-	     req->TransferLength);
+	printk(KERN_DEBUG "iflash2+_mtd: flash_read(0x%p, 0x%ld, "
+	       "0x%p, 0x%lx, 0x%lx)\n", link, req->MediaID, buf,
+	       req->SrcCardOffset, req->TransferLength);
 #endif
 
     flash = (flash_region_t *)(req->MediaID);
@@ -744,7 +718,7 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
 	    (req->SrcCardOffset / region->BlockSize)) {
 #ifdef PCMCIA_DEBUG
 	    if (pc_debug > 1)
-		kmsg("delaying read...\n");
+		printk(KERN_DEBUG "iflash2+_mtd: delaying read...\n");
 #endif
 	    req->Status = MTD_WAITREQ;
 	    return CS_BUSY;
@@ -774,9 +748,9 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
 	nb = (from+length > WINDOW_SIZE) ? WINDOW_SIZE-from : length;
 
 	if (req->Function & MTD_REQ_KERNEL)
-	    memcpy(buf, &dev->Base[from], nb);
+	    memcpy_frompc(buf, &dev->Base[from], nb);
 	else
-	    memcpy_tofs(buf, &dev->Base[from], nb);
+	    memcpy_frompc_tofs(buf, &dev->Base[from], nb);
 	
 	buf += nb;
 	from = 0;
@@ -786,9 +760,10 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
 #ifdef PCMCIA_DEBUG
     time = uticks() - time;
     if (pc_debug && (time < 10000000))
-	kmsg("read complete, time = %ld, avg = %ld ns/word, rate = "
-	     "%ld kb/sec", time, time*2000/req->TransferLength,
-	     req->TransferLength*977/time);
+	printk(KERN_DEBUG "iflash2+_mtd: read complete, time = %ld,"
+	       " avg = %ld ns/word, rate = %ld kb/sec\n", time,
+	       time*2000/req->TransferLength,
+	       req->TransferLength*977/time);
 #endif
     
 done:
@@ -819,10 +794,14 @@ static int basic_write(struct wait_queue **queue, char *esr, char *dest,
     
     /* Fix for mis-aligned writes */
     if ((u_long)dest & 1) {
-	kmsg("odd address fixup at 0x%p...", dest);
+#ifdef PCMCIA_DEBUG
+	if (pc_debug)
+	    printk(KERN_DEBUG "iflash2+_mtd: odd address fixup "
+		   "at 0x%p...\n", dest);
+#endif
 	ret = page_setup(queue, (u_short *)esr, (u_short *)dest, 1);
 	if (ret != CS_SUCCESS) return ret;
-	*dest = (is_krnl) ? (*buf) : get_fs_byte(buf);
+	writeb((is_krnl) ? (*buf) : get_fs_byte(buf), dest);
 	ret = page_write((u_short *)esr, (u_short *)dest, 1);
 	if (ret != CS_SUCCESS) return ret;
 	dest++; buf++; nb--;
@@ -837,9 +816,9 @@ static int basic_write(struct wait_queue **queue, char *esr, char *dest,
 	if (ret != CS_SUCCESS) return ret;
 	
 	if (is_krnl)
-	    memcpy(dest, buf, npb);
+	    memcpy_topc(dest, buf, npb);
 	else
-	    memcpy_fromfs(dest, buf, npb);
+	    memcpy_fromfs_topc(dest, buf, npb);
 	
 	ret = page_write((u_short *)esr, (u_short *)dest, npb);
 	if (ret != CS_SUCCESS) return ret;
@@ -875,9 +854,9 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("flash_write(0x%p, 0x%lx, 0x%p, 0x%lx, 0x%lx)",
-	     link, req->MediaID, buf, req->DestCardOffset,
-	     req->TransferLength);
+	printk(KERN_DEBUG "iflash2+_mtd: flash_write(0x%p, 0x%lx, "
+	       "0x%p, 0x%lx, 0x%lx)\n", link, req->MediaID, buf,
+	       req->DestCardOffset, req->TransferLength);
 #endif
 
     /* Check card write protect status */
@@ -903,7 +882,7 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
     if (flash->cell[cell].state & FLASH_ERASING) {
 #ifdef PCMCIA_DEBUG
 	if (pc_debug > 1)
-	    kmsg("delaying write...");
+	    printk(KERN_DEBUG "iflash2+_mtd: delaying write...\n");
 #endif
 	req->Status = MTD_WAITREQ;
 	return CS_BUSY;
@@ -945,7 +924,8 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 	    abort_cmd((u_short *)dev->ESRbase);
 	}
 	if (retry == retry_limit) {
-	    kmsg("write failed: too many retries!");
+	    printk(KERN_DEBUG "iflash2+_mtd: write failed: "
+		   "too many retries!\n");
 	    goto done;
 	}
 	
@@ -957,9 +937,10 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 #ifdef PCMCIA_DEBUG
     time = uticks() - time;
     if (pc_debug && (time < 10000000))
-	kmsg("write complete, time = %ld, avg = %ld us/word, rate = "
-	     "%ld kb/sec", time, time*2/req->TransferLength,
-	     req->TransferLength*977/time);
+	printk(KERN_DEBUG "iflash2+_mtd: write complete, time = %ld,"
+	       " avg = %ld us/word, rate = %ld kb/sec\n", time,
+	       time*2/req->TransferLength,
+	       req->TransferLength*977/time);
 #endif
     
 done:
@@ -990,9 +971,9 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("flash_erase(0x%p, 0x%lx, 0x%lx, 0x%lx)",
-	     link, req->MediaID, req->DestCardOffset,
-	     req->TransferLength);
+	printk(KERN_DEBUG "iflash2+_mtd: flash_erase(0x%p, 0x%lx, "
+	       "0x%lx, 0x%lx)\n", link, req->MediaID,
+	       req->DestCardOffset, req->TransferLength);
 #endif
 
     flash = (flash_region_t *)(req->MediaID);
@@ -1006,7 +987,7 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 	if (flash->cell[i].state & (FLASH_ERASING|FLASH_PENDING)) {
 #ifdef PCMCIA_DEBUG
 	    if (pc_debug > 1)
-		kmsg("delaying erase...\n");
+		printk(KERN_DEBUG "iflash2+_mtd: delaying erase...\n");
 #endif
 	    req->Status = MTD_WAITREQ;
 	    return CS_BUSY;
@@ -1054,7 +1035,8 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 	    goto done;
 	else if (ret != CS_BUSY) {
 	    if (++flash->cell[i].erase_retries > retry_limit) {
-		kmsg("erase failed: too many retries!");
+		printk(KERN_NOTICE "iflash2+_mtd: erase failed: "
+		       "too many retries!\n");
 		goto done;
 	    }
 	    else {
@@ -1067,7 +1049,7 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 
     /* If the request is not complete, has it taken too long? */
     if (jiffies > flash->cell[i].erase_time + erase_limit) {
-	kmsg("erase timed out!");
+	printk(KERN_NOTICE "iflash2+_mtd: erase timed out!\n");
 	log_esr(NULL, (u_short *)dev->Base);
 	abort_cmd((u_short *)dev->Base);
 	ret = CS_GENERAL_FAILURE;
@@ -1080,8 +1062,8 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
 done:
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
-	kmsg("erase complete, time = %ld",
-	     jiffies - flash->cell[i].erase_time);
+	printk(KERN_DEBUG "iflash2+_mtd: erase complete, time = %ld\n",
+	       jiffies - flash->cell[i].erase_time);
 #endif
     flash->cell[i].state &= ~(FLASH_ERASING|FLASH_PENDING);
     reset_block((u_short *)dev->ESRbase);
@@ -1147,7 +1129,7 @@ static int flash_event(event_t event, int priority,
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 1)
-	kmsg("flash_event(%x)", event);
+	printk(KERN_DEBUG "iflash2+_mtd: flash_event(%x)\n", event);
 #endif
     
     switch (event) {
@@ -1155,7 +1137,7 @@ static int flash_event(event_t event, int priority,
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
-	    link->release.expires = 5;
+	    link->release.expires = RUN_AT(HZ/20);
 	    add_timer(&link->release);
 	}
 	break;
@@ -1191,21 +1173,19 @@ static int flash_event(event_t event, int priority,
 
 /*====================================================================*/
 
-#ifdef MODULE
-char kernel_version[] = UTS_RELEASE;
-
-int init_module(void)
+int init_iflash2X_mtd(void)
 {
     servinfo_t serv;
     
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
-	printk(version);
+	printk(KERN_INFO "%s\n", version);
 #endif
     
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
-	kmsg("Card Services release does not match!");
+	printk(KERN_NOTICE "iflash2+_mtd: Card Services release "
+	       "does not match!\n");
 	return -1;
     }
     
@@ -1214,9 +1194,13 @@ int init_module(void)
     return 0;
 }
 
+#ifdef MODULE
 void cleanup_module(void)
 {
-    kmsg("unloading");
+#ifdef PCMCIA_DEBUG
+    if (pc_debug)
+	printk(KERN_DEBUG "iflash2+_mtd: unloading\n");
+#endif
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
 	flash_detach(dev_list);
