@@ -55,11 +55,21 @@ static int major_dev = 0;
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"pcmem_cs.c 1.63 1996/05/28 15:16:02 (David Hinds)";
+"pcmem_cs.c 1.64 1996/09/15 04:13:12 (David Hinds)";
 #endif
 
 /*====================================================================*/
 
+/* Parameters that can be set with 'insmod' */
+
+/* 1 = do 16-bit transfers, 0 = do 8-bit transfers */
+static int word_width = 1;
+
+/* Speed of memory accesses, in ns */
+static int mem_speed = 0;
+
+/*====================================================================*/
+
 /* Maximum number of separate memory devices we'll allow */
 #define MAX_DEV		8
 
@@ -253,7 +263,7 @@ static void get_size(dev_link_t *link, pcmem_dev_t *dev)
 {
     modwin_t mod;
     memreq_t mem;
-    char *buf = kmalloc(SIZE_NSTEP, GFP_KERNEL);
+    u_char *buf = kmalloc(SIZE_NSTEP, GFP_KERNEL);
     int s, t, ret;
 
     dev->size = 0;
@@ -324,9 +334,12 @@ static void pcmem_config(dev_link_t *link)
 
     do {
 	/* Allocate a 4K memory window */
-	req.Attributes = WIN_DATA_WIDTH_16;
+	if (word_width)
+	    req.Attributes = WIN_DATA_WIDTH_16;
+	else
+	    req.Attributes = WIN_DATA_WIDTH_8;
 	req.Base = NULL; req.Size = WINDOW_SIZE;
-	req.AccessSpeed = 0;
+	req.AccessSpeed = mem_speed;
 	link->win = (window_handle_t)link->handle;
 	ret = CardServices(RequestWindow, &link->win, &req);
 	if (ret != 0) {
