@@ -36,7 +36,7 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"serial_cs.c 1.71 1996/08/06 03:14:00 (David Hinds)";
+"serial_cs.c 1.73 1996/10/03 02:06:47 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -53,22 +53,30 @@ static int do_sound = 1;
 
 /* Table of multifunction card ID's */
 
-typedef u_short multi_id_t[2];
+typedef struct {
+    u_short	manfid;
+    u_short	prodid;
+    int		multi;		/* 1 = multifunction, 2 = dual-port */
+    int		slave;		/* 1 = not compliant with MFC spec */
+} multi_id_t;
 
 static multi_id_t multi_id[] = {
-    { MANFID_IBM, PRODID_IBM_HOME_AND_AWAY },
-    { MANFID_3COM, PRODID_3COM_3C562 },
-    { MANFID_LINKSYS, PRODID_LINKSYS_PCMLM28 },
-    { MANFID_OSITECH, PRODID_OSITECH_JACK_144 },
-    { MANFID_OSITECH, PRODID_OSITECH_JACK_288 },
-    { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_VARIOUS },
-    { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_EM3288 }
+    { MANFID_IBM, PRODID_IBM_HOME_AND_AWAY, 1, 0 },
+    { MANFID_3COM, PRODID_3COM_3C562, 1, 0 },
+    { MANFID_LINKSYS, PRODID_LINKSYS_PCMLM28, 1, 1 },
+    { MANFID_OSITECH, PRODID_OSITECH_JACK_144, 1, 1 },
+    { MANFID_OSITECH, PRODID_OSITECH_JACK_288, 1, 1 },
+    { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_VARIOUS, 1, 1 },
+    { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_EM3288, 1, 1 },
+    { MANFID_QUATECH, PRODID_QUATECH_DUAL_RS232, 2, 0 }
 };
 #define MULTI_COUNT (sizeof(multi_id)/sizeof(multi_id_t))
 
 typedef struct serial_info_t {
     int		ndev;
+    int		multi;
     int		slave;
+    int		manfid;
     dev_node_t	node[2];
     int		line[2];
 } serial_info_t;
@@ -204,6 +212,185 @@ static void serial_detach(dev_link_t *link)
     
 } /* serial_detach */
 
+/*====================================================================*/
+
+static int setup_serial(serial_info_t *info, ioaddr_t port, int irq)
+{
+    struct serial_struct serial;
+    int line;
+    
+    serial.port = port;
+    serial.irq = irq;
+    serial.flags = ASYNC_SKIP_TEST;
+#ifdef ASYNC_SHARE_IRQ
+    serial.flags |= (info->multi) ? ASYNC_SHARE_IRQ : 0;
+#endif
+    line = register_serial(&serial);
+    if (line < 0) {
+	printk(KERN_NOTICE "serial_cs: register_serial() at 0x%04x, "
+	       "irq %d failed\n", serial.port, serial.irq);
+	return -1;
+    }
+    
+    info->line[info->ndev] = line;
+    sprintf(info->node[info->ndev].dev_name, "ttyS%d", line);
+    info->node[0].major = TTY_MAJOR;
+    info->node[0].minor = 0x40+line;
+    if (info->ndev > 0)
+	info->node[info->ndev-1].next = &info->node[info->ndev];
+    info->ndev++;
+    
+    return 0;
+}
+
+/*====================================================================*/
+
+static int simple_config(dev_link_t *link)
+{
+    static ioaddr_t base[5] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8, 0x0 };
+    client_handle_t handle = link->handle;
+    serial_info_t *info = link->priv;
+    tuple_t tuple;
+    u_short buf[128];
+    cisparse_t parse;
+    cistpl_cftable_entry_t *cf = &parse.cftable_entry;
+    int i, j;
+    
+    /* First pass: look for a config entry that looks normal. */
+    tuple.TupleData = (cisdata_t *)buf;
+    tuple.TupleOffset = 0; tuple.TupleDataMax = 255;
+    tuple.Attributes = 0;
+    tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+    i = CardServices(GetFirstTuple, handle, &tuple);
+    while (i == CS_SUCCESS) {
+	i = CardServices(GetTupleData, handle, &tuple);
+	if (i != CS_SUCCESS) break;
+	i = CardServices(ParseTuple, handle, &tuple, &parse);
+	if ((i == CS_SUCCESS) && (cf->io.nwin > 0) &&
+	    ((cf->io.win[0].base & 0xf) == 8)) {
+	    link->conf.ConfigIndex = cf->index;
+	    link->io.BasePort1 = cf->io.win[0].base;
+	    i = CardServices(RequestIO, link->handle, &link->io);
+	    if (i == CS_SUCCESS) break;
+	}
+	i = CardServices(GetNextTuple, handle, &tuple);
+    }
+    
+    /* Second pass: try to find an entry that isn't picky about
+       its base address, then try to grab any standard serial port
+       address, and finally try to get any free port. */
+    if (i != CS_SUCCESS) {
+	i = CardServices(GetFirstTuple, handle, &tuple);
+	while (i == CS_SUCCESS) {
+	    i = CardServices(GetTupleData, handle, &tuple);
+	    if (i != CS_SUCCESS) break;
+	    i = CardServices(ParseTuple, handle, &tuple, &parse);
+	    if ((i == CS_SUCCESS) && (cf->io.nwin > 0) &&
+		((cf->io.flags & CISTPL_IO_LINES_MASK) <= 3)) {
+		link->conf.ConfigIndex = cf->index;
+		for (j = 0; j < 5; j++) {
+		    link->io.BasePort1 = base[j];
+		    i = CardServices(RequestIO, link->handle,
+				     &link->io);
+		    if (i == CS_SUCCESS) break;
+		}
+		if (i == CS_SUCCESS) break;
+	    }
+	    i = CardServices(GetNextTuple, handle, &tuple);
+	}
+    }
+    if (i != CS_SUCCESS) {
+	cs_error(RequestIO, i);
+	return -1;
+    }
+    
+    i = CardServices(RequestIRQ, link->handle, &link->irq);
+    if (i != CS_SUCCESS) {
+	cs_error(RequestIRQ, i);
+	link->irq.AssignedIRQ = 0;
+    }
+    if (info->multi && (info->manfid == MANFID_3COM))
+	link->conf.ConfigIndex &= ~(0x08);
+    i = CardServices(RequestConfiguration, link->handle, &link->conf);
+    if (i != CS_SUCCESS) {
+	cs_error(RequestConfiguration, i);
+	return -1;
+    }
+
+    return setup_serial(info, link->io.BasePort1, link->irq.AssignedIRQ);
+}
+
+static int slave_config(dev_link_t *link)
+{
+    client_handle_t handle = link->handle;
+    serial_info_t *info = link->priv;
+    config_info_t config;
+    ioaddr_t port;
+    int i;
+
+    /* If the card is already configured, look up the port and irq */
+    i = CardServices(GetConfigurationInfo, handle, &config);
+    if ((i == CS_SUCCESS) &&
+	(config.Attributes & CONF_VALID_CLIENT)) {
+	if (config.BasePort2 != 0)
+	    port = config.BasePort2;
+	else
+	    /* for Ositech cards */
+	    port = config.BasePort1 + 0x28;
+	info->slave = 1;
+	return setup_serial(info, port, config.AssignedIRQ);
+    } else
+	return simple_config(link);
+}
+
+static int dual_config(dev_link_t *link)
+{
+    client_handle_t handle = link->handle;
+    serial_info_t *info = link->priv;
+    tuple_t tuple;
+    u_short buf[128];
+    cisparse_t parse;
+    cistpl_cftable_entry_t *cf = &parse.cftable_entry;
+    int i;
+
+    link->io.NumPorts1 = 16;
+    tuple.TupleData = (cisdata_t *)buf;
+    tuple.TupleOffset = 0; tuple.TupleDataMax = 255;
+    tuple.Attributes = 0;
+    tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+    i = CardServices(GetFirstTuple, handle, &tuple);
+    while (i == CS_SUCCESS) {
+	i = CardServices(GetTupleData, handle, &tuple);
+	if (i != CS_SUCCESS) break;
+	i = CardServices(ParseTuple, handle, &tuple, &parse);
+	if ((i == CS_SUCCESS) && (cf->io.nwin == 1)) {
+	    link->conf.ConfigIndex = cf->index;
+	    i = CardServices(RequestIO, link->handle, &link->io);
+	    if (i == CS_SUCCESS) break;
+	}
+	i = CardServices(GetNextTuple, handle, &tuple);
+    }
+    if (i != CS_SUCCESS) {
+	cs_error(RequestIO, i);
+	return -1;
+    }
+    
+    i = CardServices(RequestIRQ, link->handle, &link->irq);
+    if (i != CS_SUCCESS) {
+	cs_error(RequestIRQ, i);
+	link->irq.AssignedIRQ = 0;
+    }
+    i = CardServices(RequestConfiguration, link->handle, &link->conf);
+    if (i != CS_SUCCESS) {
+	cs_error(RequestConfiguration, i);
+	return -1;
+    }
+
+    setup_serial(info, link->io.BasePort1, link->irq.AssignedIRQ);
+    setup_serial(info, link->io.BasePort1+8, link->irq.AssignedIRQ);
+    return 0;
+}
+
 /*======================================================================
 
     serial_config() is scheduled to run after a CARD_INSERTION event
@@ -222,9 +409,7 @@ void serial_config(dev_link_t *link)
     tuple_t tuple;
     u_short buf[128];
     cisparse_t parse;
-    struct serial_struct serial;
-    int i, j, line, multi = 0, manfid = 0;
-    static ioaddr_t base[5] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8, 0x0 };
+    int i;
 
     sti();
     handle = link->handle;
@@ -249,108 +434,36 @@ void serial_config(dev_link_t *link)
     link->conf.ConfigBase = parse.config.base;
     link->conf.Present = parse.config.rmask[0];
     
+    /* Configure card */
+    link->state |= DEV_CONFIG;
+
     /* Is this a multifunction card? */
     tuple.DesiredTuple = CISTPL_MANFID;
     tuple.Attributes = TUPLE_RETURN_COMMON;
     if ((CardServices(GetFirstTuple, handle, &tuple) == CS_SUCCESS) &&
 	(CardServices(GetTupleData, handle, &tuple) == CS_SUCCESS)) {
-	manfid = buf[0];
+	info->manfid = buf[0];
 	for (i = 0; i < MULTI_COUNT; i++)
-	    if ((buf[0] == multi_id[i][0]) && (buf[1] == multi_id[i][1]))
+	    if ((buf[0] == multi_id[i].manfid) &&
+		(buf[1] == multi_id[i].prodid))
 		break;
-	if (i < MULTI_COUNT)
-	    multi = 1;
-    }
-    
-    /* Configure card */
-    link->state |= DEV_CONFIG;
-
-    tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
-    tuple.Attributes = 0;
-    
-    /* Cards that don't follow the multifunction spec: if they are
-       already configured, look up the serial port base address. */
-    if (multi && ((manfid == MANFID_OSITECH) ||
-		  (manfid == MANFID_LINKSYS) ||
-		  (manfid == MANFID_MEGAHERTZ))) {
-	config_info_t config;
-	i = CardServices(GetConfigurationInfo, handle, &config);
-	if ((i == CS_SUCCESS) &&
-	    (config.Attributes & CONF_VALID_CLIENT)) {
-	    if (config.BasePort2 != 0)
-		serial.port = config.BasePort2;
-	    else
-		serial.port = config.BasePort1 + 0x28;
-	    serial.irq = config.AssignedIRQ;
-	    info->slave = 1;
-	} else
-	    multi = 0;
-    }
-    
-    if (!info->slave) {
-	cistpl_cftable_entry_t *cf = &parse.cftable_entry;
-
-	/* First pass: look for a config entry that looks normal. */
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	while (i == CS_SUCCESS) {
-	    i = CardServices(GetTupleData, handle, &tuple);
-	    if (i != CS_SUCCESS) break;
-	    i = CardServices(ParseTuple, handle, &tuple, &parse);
-	    if ((i == CS_SUCCESS) && (cf->io.nwin > 0) &&
-		((cf->io.win[0].base & 0xf) == 8)) {
-		link->conf.ConfigIndex = cf->index;
-		link->io.BasePort1 = cf->io.win[0].base;
-		i = CardServices(RequestIO, link->handle, &link->io);
-		if (i == CS_SUCCESS) break;
-	    }
-	    i = CardServices(GetNextTuple, handle, &tuple);
+	if (i < MULTI_COUNT) {
+	    info->multi = multi_id[i].multi;
+	    info->slave = multi_id[i].slave;
 	}
-
-	/* Second pass: try to find an entry that isn't picky about
-	   its base address, then try to grab any standard serial port
-	   address, and finally try to get any free port. */
-	if (i != CS_SUCCESS) {
-	    i = CardServices(GetFirstTuple, handle, &tuple);
-	    while (i == CS_SUCCESS) {
-		i = CardServices(GetTupleData, handle, &tuple);
-		if (i != CS_SUCCESS) break;
-		i = CardServices(ParseTuple, handle, &tuple, &parse);
-		if ((i == CS_SUCCESS) && (cf->io.nwin > 0) &&
-		    ((cf->io.flags & CISTPL_IO_LINES_MASK) <= 3)) {
-		    link->conf.ConfigIndex = cf->index;
-		    for (j = 0; j < 5; j++) {
-			link->io.BasePort1 = base[j];
-			i = CardServices(RequestIO, link->handle,
-					 &link->io);
-			if (i == CS_SUCCESS) break;
-		    }
-		    if (i == CS_SUCCESS) break;
-		}
-		i = CardServices(GetNextTuple, handle, &tuple);
-	    }
-	}
-
-	CS_EXIT_TEST(i, RequestIO, config_failed);
-    
-	i = CardServices(RequestIRQ, link->handle, &link->irq);
-	if (i != CS_SUCCESS) {
-	    cs_error(RequestIRQ, i);
-	    link->irq.AssignedIRQ = 0;
-	}
-	if (multi && (manfid == MANFID_3COM))
-	    link->conf.ConfigIndex &= ~(0x08);
-	i = CardServices(RequestConfiguration, link->handle, &link->conf);
-	CS_EXIT_TEST(i, RequestConfiguration, config_failed);
-	serial.port = link->io.BasePort1;
-	serial.irq = link->irq.AssignedIRQ;
     }
 
-    serial.flags = ASYNC_SKIP_TEST;
-#ifdef ASYNC_SHARE_IRQ
-    serial.flags |= (multi) ? ASYNC_SHARE_IRQ : 0;
-#endif
+    if (info->multi == 2)
+	dual_config(link);
+    else if (info->slave)
+	slave_config(link);
+    else
+	simple_config(link);
     
-    if (manfid == MANFID_IBM) {
+    if (info->ndev == 0)
+	goto config_failed;
+    
+    if (info->manfid == MANFID_IBM) {
 	conf_reg_t reg = { 0, CS_READ, 0x800, 0 };
 	i = CardServices(AccessConfigurationRegister,
 			 link->handle, &reg);
@@ -361,19 +474,7 @@ void serial_config(dev_link_t *link)
 			 link->handle, &reg);
 	CS_EXIT_TEST(i, AccessConfigurationRegister, config_failed);
     }
-	
-    line = register_serial(&serial);
-    if (line < 0) {
-	printk(KERN_NOTICE "serial_cs: register_serial() at 0x%04x, "
-	       "irq %d failed\n", serial.port, serial.irq);
-	goto config_failed;
-    }
-    
-    info->ndev = 1;
-    sprintf(info->node[0].dev_name, "ttyS%d", line);
-    info->node[0].major = TTY_MAJOR;
-    info->node[0].minor = 0x40+line;
-    info->line[0] = line;
+
     link->dev = &info->node[0];
     link->state &= ~DEV_CONFIG_PENDING;
     return;
@@ -425,8 +526,8 @@ void serial_release(u_long arg)
     
 ======================================================================*/
 
-int serial_event(event_t event, int priority,
-		event_callback_args_t *args)
+static int serial_event(event_t event, int priority,
+			event_callback_args_t *args)
 {
     dev_link_t *link = args->client_data;
     serial_info_t *info = link->priv;
