@@ -29,7 +29,7 @@
  *
  * ISA version modified to support PCMCIA by Anthony Joseph (adj@lcs.mit.edu)
  *
- * /usr/src/pc/modules/RCS/wavelan_cs.c,v 1.6 1996/08/29 04:41:46 root Exp
+ * /usr/src/pc/modules/RCS/wavelan_cs.c,v 1.10 1996/10/01 14:24:19 root Exp
  *
  ****************************************************************************
  *   Copyright 1995
@@ -57,6 +57,18 @@
  *
  * RCS revisions:
  * wavelan_cs.c,v
+ * Revision 1.10  1996/10/01 14:24:19  root
+ * *** empty log message ***
+ *
+ * Revision 1.9  1996/09/21 15:23:05  root
+ * *** empty log message ***
+ *
+ * Revision 1.8  1996/09/08 22:41:51  root
+ * *** empty log message ***
+ *
+ * Revision 1.7  1996/09/08 22:38:07  root
+ * *** empty log message ***
+ *
  * Revision 1.6  1996/08/29 04:41:46  root
  * *** empty log message ***
  *
@@ -123,15 +135,15 @@
  *	 James Ashton (jaa101@syseng.anu.edu.au),
  *	 Ajay Bakre (bakre@paul.rutgers.edu),
  *	 Donald Becker (becker@super.org),
- *	 Loeke Brederveld (Loeke.Brederveld@Utrecht.NCR.com),
+ *	 Loeke Brederveld (brederveld@lucent.com),
  *	 Allan Creighton (allanc@cs.su.oz.au),
  *	 Matthew Geier (matthew@cs.su.oz.au),
  *	 Remo di Giovanni (remo@cs.su.oz.au),
  *	 Mark Hagan (mhagan@wtcpost.daytonoh.NCR.COM),
- *	 Jan Hoogendoorn (JHOOGEND@wcnd.utrecht.attgis.com),
+ *	 Jan Hoogendoorn (c/o marteijn@lucent.com),
  *       Bruce Janson (bruce@cs.su.oz.au)
  *	 Anders Klemets (klemets@paul.rutgers.edu),
- *	 Marc Meertens (Marc.Meertens@Utrecht.NCR.com),
+ *	 Marc Meertens (mmeertens@lucent.com),
  *	 Robert Morris (rtm@das.harvard.edu),
  *	 Ian Parkin (ian@cs.su.oz.au),
  *	 John Rosenberg (johnr@cs.su.oz.au),
@@ -195,7 +207,7 @@ static int pc_debug = PCMCIA_DEBUG;
 #else
 static int pc_debug = 0;
 #endif
-static const char *version = "wavelan_cs.c,v 1.6 1996/08/29 04:41:46 root Exp\n";
+static const char *version = "wavelan_cs.c,v 1.10 1996/10/01 14:24:19 root Exp\n";
 
 /*====================================================================*/
 
@@ -2033,23 +2045,17 @@ static int wv_start(struct device *dev)
     
     irq2dev_map[dev->irq] = dev;
 
-    dev->tbusy = 0;   
+    dev->tbusy = 0; dev->start = 1;
 
     x = wv_splhi();
     if ((r = wv_hw_reset(dev)) != -1)
-    {
 	dev->interrupt = 0;
-	dev->start = 1;
-    }
     wv_splx(x);
 
     if (r == -1) 
     {
-#ifdef SA_SHIRQ
-    	free_irq(dev->irq,NULL);
-#else
-    	free_irq(dev->irq);
-#endif
+	dev->start = 0;
+    	FREE_IRQ(dev->irq);
     	irq2dev_map[dev->irq] = (device *)0;
     	return(-EAGAIN);
     }
@@ -2280,7 +2286,7 @@ static void wavelan_config(dev_link_t *link)
     tuple_t tuple;
     cisparse_t parse;
     struct device *dev;
-    int i, j;
+    int i;
     u_char buf[64];
     win_req_t req;
     memreq_t mem;
@@ -2652,8 +2658,7 @@ static void wv_interrupt(device *dev)
       	    return;
     	}
 	/* This should tell us if the card isn't really there */
-	if (!dev->start ||
-	    (status0 & SR0_EVENT_MASK == SR0_DIAGNOSE_FAILED)) {
+	if ((dev->start == 0) || (status0 == 0xff)) {
 #ifdef WAVELAN_DEBUG
 	    if (wavelan_debug > 1)
 		printk("%s: interrupt from dead card\n", dev->name);
